/* Callback for item selection and state changes in the  */
/* file selector widget.                                 */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 2.01  */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"
#include "playlist.h"


int
SaveSelected( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

{
	PtFileSelCallback_t	*selection;
	PtFileSelItem_t		*item;
	static struct timeval   last_click_time = { 0, 0 };
	static char		*last_filename = "";
	struct timeval          now, diff;

	selection = (PtFileSelCallback_t *)cbinfo->cbdata;

	if (cbinfo->reason == Pt_CB_FS_SELECTION)
	{
		item = (PtFileSelItem_t *)selection->item;

		if (item->type == Pt_FS_FILE && cbinfo->reason_subtype == 1)
		{
			char	*filename = strrchr(item->fullpath, '/');

			if (filename == NULL)
				filename = item->fullpath;
			else
				filename++;

			// Is this a double-click?
			gettimeofday(&now, NULL);
			timersub(&now, &last_click_time, &diff);
			if (diff.tv_sec == 0 &&
			    diff.tv_usec < DOUBLECLICK_TIME_MS * 1000)
			{
				// If it's a different file than the previous
				// click, don't treat it as a double-click.
				if (! strcmp(filename, last_filename))
				{
					BtnSaveOK(NULL, NULL, NULL);
					PtUnrealizeWidget(ABW_SaveDialog);
					return (Pt_CONTINUE);
				}
			}

			last_click_time = now;

			PtSetResource(ABW_SaveFileText, Pt_ARG_TEXT_STRING,
					filename, 0);
			PtGetResource(ABW_SaveFileText, Pt_ARG_TEXT_STRING,
					&last_filename, 0);

		}
	}
	
	if (cbinfo->reason == Pt_CB_FS_STATE)
	{
		if (selection->reason == Pt_FS_STATE_START)
		{
			PtSetResource(widget, Pt_ARG_FLAGS, Pt_BLOCKED,
					Pt_BLOCKED);
		}
		else
		{
			char	*path;

			PtGetResource(ABW_SaveFileSel, Pt_ARG_FS_ROOT_DIR,
					&path, 0);
			PtSetResource(ABW_SaveDialog, Pt_ARG_WINDOW_TITLE,
					path, 0);
			PtSetResource(widget, Pt_ARG_FLAGS, ~Pt_BLOCKED,
					Pt_BLOCKED);
		}
	}

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	return( Pt_CONTINUE );

}

