#ifndef __DECODERS_H__
#define __DECODERS_H__ 1


/*
 * Decoder modules have to support some common operations.  Internal
 * implementation is completely decoder-dependent.
 */
struct decoder {
	char	*name;			// Name of decoder module, e.g. madplay

	/*
	 * NULL-terminated list of supported file extensions.  (Case
	 * insensitive.)
	 */
	char	**extensions;

	/*
	 * Stops playing the current song.
	 */
	void	(*stop_playing)(void);

	/*
	 * Stops playing any existing song if necessary, and starts playing
	 * a new one.  If you're forking a separate process and need to
	 * wake up when it spits something out, you can register its file
	 * descriptor with Photon using PtAppAddFd().  Returns 0 on successful
	 * start of playback.
	 */
	int	(*start_playing)(char *filename);

	/*
	 * Returns nonzero if there is currently something playing or paused.
	 */
	int	(*is_playing)(void);

	/*
	 * Toggles pause mode on and off.
	 */
	void	(*toggle_pause)(void);

	/*
	 * Jumps to a particular time in the song.  This callback may be null
	 * in which case the song-position slider is disabled.
	 */
	void	(*jump_to_time)(int seconds);
};

/* decoders.c has the list of available decoders. */
extern struct decoder *decoders[];
extern struct decoder *decoder;

/*
 * Decoders may call the following helper functions to update the UI:
 */

/* Sets the current track name. */
void set_track_name(char *value);

/* Sets the current album name. */
void set_album_name(char *value);

/* Sets the current artist name. */
void set_artist_name(char *value);

/* Sets the current playback position in seconds. */
void set_current_time(int seconds);

/* Sets the total track time in seconds. */
void set_total_time(int seconds);

/* Notifies the application that the track is done playing. */
void track_is_done(void);

/* Prints verbose output when spawn() fails. */
void spawn_error(const char *program);

extern struct decoder *find_decoder(char *filename);

#endif /* __DECODERS_H__ */
