/*
 * Photon IPC module.  Modified version of ipc.c by Jim Mazrimas from ACID,
 * http://www.timemocksme.com/acid
 */

/* Standard headers */
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Local headers */
#include "ablibs.h"
#include "abimport.h"
#include "proto.h"

enum MsgType 
{
	MSGTYPE_RESTORE
};

struct Msg
{
	int nothing;
};

enum ReplyType 
{
	REPTYPE_SUCCESS, 
	REPTYPE_BADMSG
};

struct Reply 
{
	enum ReplyType status;
};

// Handle a message from a client:
static PtConnectionMsgFunc_t msghandler;

static void const *msghandler(PtConnectionServer_t *connection, void *data,
			unsigned long type, void const *msgptr, unsigned msglen,
			unsigned *reply_len) 
{
	struct Msg const *msg = (struct Msg const*) msgptr;
	static struct Reply reply;

	reply.status = REPTYPE_SUCCESS;
	if (type == MSGTYPE_RESTORE) 
	{
		PtWindowToFront(ABW_PhJuke);
		PtConnectionReply( connection, sizeof(reply), &reply );
	}
	
	*reply_len = sizeof(reply);
	return &reply;
}

// Set up a new connection:
static PtConnectorCallbackFunc_t connector_callback;

static void connector_callback(PtConnector_t *connector,
			PtConnectionServer_t *connection, void *data )
{
	static const PtConnectionMsgHandler_t handlers = { 0, msghandler };
  
	if (PtConnectionAddMsgHandlers( connection, &handlers, 1 ) != 0 ) 
	{
		fputs( "Unable to set up connection handler\n", stderr );
		PtConnectionServerDestroy( connection );
	}
}



int phjuke_is_running(void)
{
	PtConnectionClient_t *inst;
	static const char connectorName[] = "www.midwinter.com/phjuke";
	
	while (PtConnectorCreate(connectorName, connector_callback, 0) == NULL)
	{
		// another instance of the app must be already running
		// Send it a message to restore (from hidden state) and then
		// exit myself
		inst = PtConnectionFindName(connectorName, 0, 0);
		if (inst) 
		{
			struct Reply reply;
			struct Msg msg = { 0 };
			int result;
			result = PtConnectionSend( inst, MSGTYPE_RESTORE, &msg,
					&reply, sizeof(msg), sizeof(reply));
			PtConnectionClientDestroy(inst);
			if (result == -1)
				fprintf(stderr, "Error sending message to "
					"existing instance of PhJuke\n");
			return 1;
		}
	}

	// Since PtConnectorCreate() has succeeded, we're the only instance
	// of the app running
	return 0;
}
