/*
 * Directory-mapping functions.
 */
#include <stdio.h>
#include <string.h>
#include "playlist.h"
#include "gfuncs.h"

/*
 * Mapped directory, used for M3U files with absolute paths that need to
 * be mapped to local mount points.
 */
typedef struct _map_dir MAP_DIR;
struct _map_dir {
	char	*from;
	char	*to;
	MAP_DIR	*next;
};

static MAP_DIR *map_dirs = NULL;


/*
 * Adds an equals-sign-separated directory pair to the map list.
 */
void
add_map_dir(char *pair)
{
	MAP_DIR	*dir = gcalloc(sizeof(MAP_DIR));

	dir->from = gstrdup(strtok(pair, "="));
	dir->to = gstrdup(strtok(NULL, "\n\r"));
	if (dir->from == NULL || dir->to == NULL)
	{
		gfree(dir);
		return;
	}

	dir->next = map_dirs;
	map_dirs = dir;
}

/*
 * Dumps a list of mapped directories to the prefs file.
 */
void
print_map_dirs(FILE *fp)
{
	MAP_DIR	*dir;

	for (dir = map_dirs; dir != NULL; dir = dir->next)
		fprintf(fp, "map-dir %s=%s\n", dir->from, dir->to);
}


/*
 * Examines a path to see if any directories need to be remapped.
 * Modifies the path in place.
 */
void
map_dir(char *path)
{
	MAP_DIR	*dir;

	for (dir = map_dirs; dir != NULL; dir = dir->next)
	{
		if (! strncasecmp(path, dir->from, strlen(dir->from)))
		{
			char	newpath[1000];

			strcpy(newpath, dir->to);
			strcat(newpath, path + strlen(dir->from));
			strcpy(path, newpath);
			break;
		}
	}
}
