/*
 * Remote control module.  PhJuke can listen on a TCP port and accept
 * commands from remote hosts.
 */
#include <stdio.h>
#include <unix.h>
#include <sys/socket.h>
#include <ctype.h>
#include <netinet/in.h>
#include <string.h>
#include "ablibs.h"
#include "abimport.h"
#include "abdefine.h"
#include "playlist.h"

static int port_num = -1;
static int listen_fd = -1;

extern char current_artist[], current_album[], current_track[];
extern int current_time, current_length;


/*
 * Closes a connected socket.
 */
void
network_close(int fd)
{
	PtAppRemoveFd(NULL, fd);
	close(fd);
}

/*
 * Called when data is available on a connected socket.
 */
int
network_read(int fd, void *dummy, unsigned type)
{
	char	buf[2000];
	int	pos;
	int	bytesread = 0;
	char	*msg = "200 OK\n";

	for (pos = 0;
	     pos < sizeof(buf) && (bytesread = read(fd, buf + pos, 1)) == 1;
	     pos++)
	{
		if (buf[pos] == '\n' || buf[pos] == '\r')
			break;
	}
	buf[pos] = '\0';

	if (bytesread < 0)
		perror("Can't read from network");
	if (bytesread <= 0)
		network_close(fd);
	if (pos == 0)
		return 0;
	
	if (! strcasecmp(buf, "play"))
		BtnPlay(NULL, NULL, NULL);
	else if (! strcasecmp(buf, "stop"))
		BtnStop(NULL, NULL, NULL);
	else if (! strcasecmp(buf, "pause"))
		BtnPause(NULL, NULL, NULL);
	else if (! strcasecmp(buf, "prev") || ! strcasecmp(buf, "back"))
		BtnPrev(NULL, NULL, NULL);
	else if (! strcasecmp(buf, "next") || ! strcasecmp(buf, "forward"))
		BtnNext(NULL, NULL, NULL);
	else if (! strncasecmp(buf, "launch", 6))
	{
		char	*listname;

		strtok(buf, " ");
		listname = strtok(NULL, "\n");
		if (listname != NULL)
			launch_file(listname, 1);
	}
	else if (! strcasecmp(buf, "clear"))
	{
		stop_playing();
		clear_list();
	}
	else if (! strncasecmp(buf, "save", 4))
	{
		char	*listname, *lastslash;

		strtok(buf, " ");
		listname = strtok(NULL, "\n");
		if (listname != NULL)
		{
			lastslash = strrchr(listname, '/');
			if (lastslash != NULL)
			{
				*lastslash = '\0';
				save_list(listname, lastslash + 1);
			}
			else
			{
				save_list(".", listname);
			}
		}
	}
	else if (! strcasecmp(buf, "status"))
	{
		sprintf(buf, "mode %s\n", is_playing() ? "play" : "stop");
		if (is_playing())
		{
			sprintf(buf + strlen(buf),
				"time %d\nlength %d\nartist %s\nalbum %s\n"
				"track %s\n",
				current_time, current_length,
				current_artist, current_album, current_track);
		}

		write(fd, buf, strlen(buf));
	}
	else if (! strcasecmp(buf, "quit"))
	{
		WinQuit(NULL, NULL, NULL);
		write(fd, msg, strlen(msg));
		PtExit(0);
	}
	else if (! strcasecmp(buf, "disconnect"))
	{
		write(fd, msg, strlen(msg));
		network_close(fd);
		return 0;
	}
	else
	{
		printf("Unrecognized network command: %s\n", buf);
		msg = "500 Unrecognized command\n";
	}

	write(fd, msg, strlen(msg));
	return 0;
}



/*
 * Called when "data" is available on the listen socket, i.e., a connection
 * is pending.
 */
int
network_accept(int fd, void *dummy, unsigned type)
{
	struct sockaddr_in	sin;
	int			sin_size = sizeof(sin);
	int			new_fd;

	new_fd = accept(fd, (struct sockaddr *)&sin, &sin_size);
	if (new_fd < 0)
	{
		perror("accept");
		return 0;
	}

	PtAppAddFd(NULL, new_fd, Pt_FD_READ, network_read, NULL);

	return 0;
}


/*
 * Sets the port number and starts listening for connections.
 */
int
network_init(int port)
{
	struct sockaddr_in	sin;

	port_num = port;

	listen_fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (listen_fd < 0)
	{
		perror("socket");
		return -1;
	}

	memset(&sin, 0, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = INADDR_ANY;
	sin.sin_port = htons(port);
	if (bind(listen_fd, (struct sockaddr *)&sin, sizeof(sin)) < 0)
	{
		perror("bind");
		return -1;
	}

	if (listen(listen_fd, 5) < 0)
	{
		perror("listen");
		return -1;
	}

	PtAppAddFd(NULL, listen_fd, Pt_FD_READ, network_accept, NULL);

	return 0;
}


/*
 * Returns the listen port number, or -1 for none.
 */
int
get_port()
{
	return port_num;
}


/*
 * Shuts down the listening socket.
 */
void
network_shutdown()
{
	if (listen_fd >= 0)
	{
		PtAppRemoveFd(NULL, listen_fd);
		close(listen_fd);

		listen_fd = -1;
	}
}
